@@ -28,6 +28,7 @@
 #include <glib/gthread.h>
 #include <glib/ghash.h>
 
+#include "galias.h"
 
 /* < private >
  * GVariantTypeInfo:
@@ -50,7 +51,7 @@
  * for "as" (note that "s" and "v" always exist in the static array).
  *
  * The trickiest part of GVariantTypeInfo (and in fact, the major reason
- * for its existence) is the storage of somewhat magical constants that
+ * for its existance) is the storage of somewhat magical constants that
  * allow for O(1) lookups of items in tuples.  This is described below.
  *
  * 'container_class' is set to 'a' or 'r' if the GVariantTypeInfo is
@@ -867,3 +868,6 @@ g_variant_type_info_assert_no_infos (void)
 {
   g_assert (g_variant_type_info_table == NULL);
 }
+
+#define __G_VARIANT_TYPE_INFO_C__
+#include "galiasdef.c"
